Attribute VB_Name = "SystemTray"
Public Declare Function Shell_NotifyIcon Lib "shell32" Alias "Shell_NotifyIconA" (ByVal dwMessage As Long, lpData As NOTIFYICONDATA) As Long
Private Declare Function GetFileVersionInfoSize Lib "version.dll" Alias "GetFileVersionInfoSizeA" (ByVal lptstrFilename As String, lpdwHandle As Long) As Long
Private Declare Function GetFileVersionInfo Lib "version.dll" Alias "GetFileVersionInfoA" (ByVal lptstrFilename As String, ByVal dwHandle As Long, ByVal dwLen As Long, lpData As Any) As Long
Private Declare Function VerQueryValue Lib "version.dll" Alias "VerQueryValueA" (pBlock As Any, ByVal lpSubBlock As String, lpBuffer As Any, nVerSize As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)

Public InTheTray As NOTIFYICONDATA

Public Type GUID
   Data1 As Long
   Data2 As Integer
   Data3 As Integer
   Data4(7) As Byte
End Type

Public Type NOTIFYICONDATA
  cbSize As Long
  hWnd As Long
  uID As Long
  uFlags As Long
  uCallbackMessage As Long
  hIcon As Long
  szTip As String * 128
  dwState As Long
  dwStateMask As Long
  szInfo As String * 256
  uTimeoutAndVersion As Long
  szInfoTitle As String * 64
  dwInfoFlags As Long
  guidItem As GUID
End Type

Public Const NIM_ADD = &H0
Public Const NIM_MODIFY = &H1
Public Const NIM_DELETE = &H2
Public Const WM_MOUSEMOVE = &H200
Public Const NIF_MESSAGE = &H1
Public Const NIF_ICON = &H2
Public Const NIF_TIP = &H4
Public Const WM_LBUTTONDBLCLK = &H203
Public Const WM_LBUTTONDOWN = &H201
Public Const WM_LBUTTONUP = &H202
Public Const WM_RBUTTONDBLCLK = &H206
Public Const WM_RBUTTONDOWN = &H204
Public Const WM_RBUTTONUP = &H205

Public Const NOTIFYICONDATA_V1_SIZE As Long = 88  'pre-5.0 structure size
Public Const NOTIFYICONDATA_V2_SIZE As Long = 488 'pre-6.0 structure size
Public Const NOTIFYICONDATA_V3_SIZE As Long = 504 '6.0+ structure size
Public NOTIFYICONDATA_SIZE As Long
Public Const APP_SYSTRAY_ID = 999 'unique identifier
Public Const NIF_INFO = &H10




Public Sub AddTrayIcon(Icon As Long, hWndID As Long, Optional ToolTip As String)
InTheTray.cbSize = Len(InTheTray)
InTheTray.hWnd = hWndID
InTheTray.uID = vbNull
InTheTray.uFlags = NIF_ICON Or NIF_TIP Or NIF_MESSAGE
InTheTray.uCallbackMessage = WM_MOUSEMOVE
InTheTray.hIcon = Icon
InTheTray.szTip = ToolTip & vbNullChar

Shell_NotifyIcon NIM_ADD, InTheTray
End Sub
Public Sub RemoveTrayIcon()
Shell_NotifyIcon NIM_DELETE, InTheTray
End Sub
Public Function TrayEvent(mouseX As Single) As String
Dim Msg As Long
Dim sFilter As String
   
Msg = mouseX / Screen.TwipsPerPixelX

Select Case Msg
    Case WM_LBUTTONDOWN
        TrayEvent = "LEFTDOWN"
    Case WM_LBUTTONUP
        TrayEvent = "LEFTUP"
    Case WM_LBUTTONDBLCLK
        TrayEvent = "LEFTDOUBLE"
    Case WM_RBUTTONDOWN
        TrayEvent = "RIGHTDOWN"
    Case WM_RBUTTONUP
        TrayEvent = "RIGHTUP"
    Case WM_RBUTTONDBLCLK
        TrayEvent = "RIGHTDOUBLE"
End Select
End Function

Public Sub TrayToolTip(Message As String)
InTheTray.szTip = Message & vbNullChar
Shell_NotifyIcon NIM_MODIFY, InTheTray
End Sub

Public Sub ChangeTrayIcon(Icon As Long)
InTheTray.hIcon = Icon
Shell_NotifyIcon NIM_MODIFY, InTheTray
End Sub

Public Sub ShellTrayModifyTip(UserText As String)

   If NOTIFYICONDATA_SIZE = 0 Then SetShellVersion
   
   With InTheTray
      .cbSize = NOTIFYICONDATA_SIZE
      .hWnd = Main.hWnd
      .uID = vbNull
      .uFlags = NIF_INFO
      .dwInfoFlags = 0
      
      'InfoTitle is the balloon tip title,
      'and szInfo is the message displayed.
      'Terminating both with vbNullChar prevents
      'the display of the unused padding in the
      'strings defined as fixed-length in NOTIFYICONDATA.
      .szInfoTitle = "User Connected !" & vbNullChar
      .szInfo = UserText & vbNullChar
   End With

   Call Shell_NotifyIcon(NIM_MODIFY, InTheTray)

End Sub


Public Sub SetShellVersion()

   Select Case True
      Case IsShellVersion(6)
         NOTIFYICONDATA_SIZE = NOTIFYICONDATA_V3_SIZE '6.0+ structure size
      
      Case IsShellVersion(5)
         NOTIFYICONDATA_SIZE = NOTIFYICONDATA_V2_SIZE 'pre-6.0 structure size
      
      Case Else
         NOTIFYICONDATA_SIZE = NOTIFYICONDATA_V1_SIZE 'pre-5.0 structure size
   End Select

End Sub


Public Function IsShellVersion(ByVal version As Long) As Boolean

  'returns True if the Shell version
  '(shell32.dll) is equal or later than
  'the value passed as 'version'
   Dim nBufferSize As Long
   Dim nUnused As Long
   Dim lpBuffer As Long
   Dim nVerMajor As Integer
   Dim bBuffer() As Byte
   
   Const sDLLFile As String = "shell32.dll"
   
   nBufferSize = GetFileVersionInfoSize(sDLLFile, nUnused)
   
   If nBufferSize > 0 Then
    
      ReDim bBuffer(nBufferSize - 1) As Byte
    
      Call GetFileVersionInfo(sDLLFile, 0&, nBufferSize, bBuffer(0))
    
      If VerQueryValue(bBuffer(0), "\", lpBuffer, nUnused) = 1 Then
         
         CopyMemory nVerMajor, ByVal lpBuffer + 10, 2
        
         IsShellVersion = nVerMajor >= version
      
      End If  'VerQueryValue
    
   End If  'nBufferSize
  
End Function
